﻿using System;
using System.Web.UI;
using System.Web.UI.WebControls;

public class EventArgs<T> : EventArgs
{
    public EventArgs(T value)
    {
        this.Value = value;
    }

    public T Value { get; private set; }
}

public partial class GridPager : UserControl
{
    public event EventHandler SavePersonalizationClicked;
    public event EventHandler EditPersonalizationClicked;
    public event EventHandler<EventArgs<int>> PageIndexChanged;
    public event EventHandler<EventArgs<int>> PageSizeChanged;

    public string GridClientId { get; set; }
    public int PageIndex { get; set; }
    public int PageSize { get; set; }
    public int PageCount { get; set; }

    public void SetContext(string gridClientId, int index, int count, int size)
    {
        this.GridClientId = gridClientId;
        this.PageIndex = index;
        this.PageCount = count;
        this.PageSize = size;
    }

    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);

        PreviousPageBtn.Visible = PageIndex > 0;
        PreviousPageBtn.Attributes.Add("OnClick",
                                       string.Format("gotoPage(\"{0}\", {1}); return false;", GridClientId, PageIndex - 1));
        PreviousPageLbl.Visible = !PreviousPageBtn.Visible;
        //PreviousPageBtn.CommandArgument = (PageIndex - 1).ToString();

        NextPageBtn.Visible = PageIndex < PageCount - 1;
        NextPageBtn.Attributes.Add("OnClick",
                                   string.Format("gotoPage(\"{0}\", {1}); return false;", GridClientId, PageIndex + 1));
        NextPageLbl.Visible = !NextPageBtn.Visible;
        //NextPageBtn.CommandArgument = (PageIndex + 1).ToString();

        //ChangePageSizeBtn.OnClientClick = string.Format("gotoPage(\"{0}\", 0)", GridClientId);
        PageNrBox.Text = (PageIndex + 1).ToString();
        GoToPageBtn.Attributes.Add("OnClick",
                                   string.Format("gotoPage(\"{0}\", getTextboxValue(this)-1)", GridClientId));

        RowsTextBox.Text = PageSize.ToString();
    }

    #region Events

    private void FireEvent(EventHandler handler)
    {
        EventHandler localhandler = handler;
        if (localhandler != null)
        {
            localhandler(this, EventArgs.Empty);
        }
    }

    private void FireEvent<T>(EventHandler<T> handler, T args) where T : EventArgs
    {
        EventHandler<T> localhandler = handler;
        if (localhandler != null)
        {
            localhandler(this, args);
        }
    }

    protected void GoToPageButton_Clicked(object sender, EventArgs e)
    {
        int page;
        if (!Int32.TryParse(this.PageNrBox.Text, out page))
            return;
        FireEvent(this.PageIndexChanged, new EventArgs<int>(page - 1));
    }

    protected void ChangePageSizeButton_Clicked(object sender, EventArgs e)
    {
        int size;
        if (!Int32.TryParse(this.RowsTextBox.Text, out size))
            return;

        FireEvent(this.PageSizeChanged, new EventArgs<int>(size));
    }

    protected void SavePersonalizationButton_Clicked(object sender, EventArgs e)
    {
        FireEvent(this.SavePersonalizationClicked);
    }

    protected void ChangePage_Clicked(object sender, CommandEventArgs e)
    {
        FireEvent(this.PageIndexChanged, new EventArgs<int>(Convert.ToInt32(e.CommandArgument)));
    }
    #endregion

    protected void EditPersonalizationButton_Clicked(object sender, EventArgs e)
    {
        FireEvent(this.EditPersonalizationClicked);
    }
}
